@@ -5,17 +5,15 @@ py_install_method_detect <- function(envname, conda = "auto") {
   if (virtualenv_exists(envname))
     return("virtualenv")
 
-  # check and prompt for miniconda
-  if (miniconda_enabled() && miniconda_installable() && !miniconda_exists())
-    miniconda_install_prompt()
-
   # try to find an existing condaenv
   if (condaenv_exists(envname, conda = conda))
     return("conda")
 
   # check to see if virtualenv or venv is available
-  python <- virtualenv_default_python()
-  if (python_has_module(python, "virtualenv") || python_has_module(python, "venv"))
+  python <- virtualenv_starter()
+  if (!is.null(python) &&
+      (python_has_module(python, "venv") ||
+       python_has_module(python, "virtualenv")))
     return("virtualenv")
 
   # check to see if conda is available
@@ -87,15 +85,39 @@ py_install <- function(packages,
 {
   check_forbidden_install("Python packages")
 
+  if (is.null(envname) && is_epheremal_venv_initialized()) {
+    if (!is.null(python_version)) {
+      stop(
+        "Python version requirements cannot be ",
+        "changed after Python has been initialized"
+      )
+    }
+    warning(
+      "An ephemeral virtual environment managed by 'reticulate' is currently in use.\n",
+      "To add more packages to your current session, call `py_require()` instead\n",
+      "of `py_install()`. Running:\n  ",
+      paste0(
+        "`py_require(", paste0(sprintf("\"%s\"", packages), collapse = ", "), ")`"
+      )
+    )
+    py_require(packages)
+    return(invisible())
+  }
+
   # if 'envname' was not provided, use the 'active' version of Python
   if (is.null(envname)) {
 
     python <- tryCatch(py_exe(), error = function(e) NULL)
     if (!is.null(python)) {
 
-      # form path to environment
+      # get information on default version of python
       info <- python_info(python)
-      envname <- info$root
+
+      # if this version of python is associated with a python environment,
+      # then set 'envname' to use that environment
+      type <- info$type %||% "unknown"
+      if (type %in% c("virtualenv", "conda"))
+        envname <- info$root
 
       # update installation method
       if (identical(info$type, "virtualenv"))
@@ -125,6 +147,7 @@ py_install <- function(packages,
       envname = envname,
       packages = packages,
       ignore_installed = pip_ignore_installed,
+      python_version = python_version,
       ...
     ),
 
@@ -156,6 +179,8 @@ py_resolve <- function(envname = NULL,
   if (is.null(envname))
     return(py_exe())
 
+  type <- match.arg(type)
+
   # if envname was supplied, try to resolve the environment path
   envpath <- if (type == "virtualenv") {
 
