@@ -36,6 +36,8 @@ install_miniconda <- function(path = miniconda_path(),
 {
   check_forbidden_install("Miniconda")
 
+  path <- path.expand(path)
+
   if (grepl(" ", path, fixed = TRUE))
     stop("cannot install Miniconda into a path containing spaces")
 
@@ -48,6 +50,11 @@ install_miniconda <- function(path = miniconda_path(),
   url <- miniconda_installer_url()
   installer <- miniconda_installer_download(url)
 
+  if (force) {
+    # miniconda installer '-u' (update) flag frequently does not work, errors.
+    miniconda_uninstall(path)
+  }
+
   # run the installer
   miniconda_installer_run(installer, update, path)
 
@@ -80,7 +87,8 @@ install_miniconda <- function(path = miniconda_path(),
 #' @export
 miniconda_update <- function(path = miniconda_path()) {
   conda <- miniconda_conda(path)
-  system2(conda, c("update", "--yes", "--name", "base", "conda"))
+  local_conda_paths(conda)
+  system2t(conda, c("update", "--yes", "--name", "base", "conda"))
 }
 
 #' Remove Miniconda
@@ -196,7 +204,8 @@ miniconda_installer_download <- function(url) {
 miniconda_installer_run <- function(installer, update, path) {
 
   args <- if (is_windows()) {
-    dir.create(path, recursive = TRUE, showWarnings = FALSE)
+    if(dir.exists(path))
+      unlink(path, recursive = TRUE)
     c(
       "/InstallationType=JustMe",
       "/AddToPath=0",
@@ -233,8 +242,17 @@ miniconda_installer_run <- function(installer, update, path) {
     on.exit(Sys.setenv(DYLD_FALLBACK_LIBRARY_PATH = old), add = TRUE)
 
   }
-
-  status <- system2(installer, args)
+  if (is_windows()) {
+    installer <- normalizePath(installer)
+    status <- system2t(installer, args)
+  }
+  if (is_unix()) {
+    ##check for bash
+    bash_path <- Sys.which("bash")
+    if (bash_path[1] == "")
+      stopf("The miniconda installer requires bash.")
+    status <- system2t(bash_path[1], c(installer, args))
+  }
   if (status != 0)
     stopf("miniconda installation failed [exit code %i]", status)
 
@@ -270,7 +288,7 @@ miniconda_path_default <- function() {
   }
 
   # otherwise, use rappdirs default
-  root <- normalizePath(rappdirs::user_data_dir(), winslash = "/", mustWork = FALSE)
+  root <- normalizePath(user_data_dir(), winslash = "/", mustWork = FALSE)
   file.path(root, "r-miniconda")
 
 }
@@ -293,11 +311,15 @@ miniconda_conda <- function(path = miniconda_path()) {
 
 miniconda_envpath <- function(env = NULL, path = miniconda_path()) {
   env <- env %||% Sys.getenv("RETICULATE_MINICONDA_ENVNAME", unset = "r-reticulate")
+
+  if(env == 'base')
+    return(path)
+
   file.path(path, "envs", env)
 }
 
 miniconda_meta_path <- function() {
-  root <- rappdirs::user_data_dir("r-reticulate")
+  root <- user_data_dir("r-reticulate")
   file.path(root, "miniconda.json")
 }
 
@@ -386,7 +408,7 @@ miniconda_python_envpath <- function() {
 
 # the version of python to use in the environment
 miniconda_python_version <- function() {
-  Sys.getenv("RETICULATE_MINICONDA_PYTHON_VERSION", unset = "3.8")
+  Sys.getenv("RETICULATE_MINICONDA_PYTHON_VERSION", unset = "3.10")
 }
 
 miniconda_python_package <- function() {
