@@ -24,22 +24,20 @@
                                           environment = character(),
                                           stringsAsFactors = FALSE)
 .globals$suppress_warnings_handlers <- list()
-.globals$class_filters <- list(
-
-  function(classes) {
-    if ("python.builtin.BaseException" %in% classes) {
-      classes <- unique(c(classes, "error", "condition"))
-    }
-    classes
-  }
-
-)
+.globals$class_filters <- list()
 .globals$py_repl_active <- FALSE
 
 is_python_initialized <- function() {
   !is.null(.globals$py_config)
 }
 
+is_epheremal_venv_initialized <- function() {
+  isTRUE(.globals$py_config$ephemeral)
+}
+
+is_python_finalized <- function() {
+  identical(.globals$finalized, TRUE)
+}
 
 ensure_python_initialized <- function(required_module = NULL) {
 
@@ -47,6 +45,9 @@ ensure_python_initialized <- function(required_module = NULL) {
   if (is_python_initialized())
     return()
 
+  if (is_python_finalized())
+    stop("py_initialize() cannot be called more than once per R session or after py_finalize(). Please start a new R session.")
+
   # notify front-end (if any) that Python is about to be initialized
   callback <- getOption("reticulate.python.beforeInitialized")
   if (is.function(callback))
@@ -66,7 +67,7 @@ ensure_python_initialized <- function(required_module = NULL) {
   remap_output_streams()
   set_knitr_python_stdout_hook()
 
-  if (is_windows() && identical(.Platform$GUI, "RStudio"))
+  if (is_windows() && ( is_rstudio() || is_positron() ))
     import("rpytools.subprocess")$patch_subprocess_Popen()
 
   # generate 'R' helper object
@@ -86,13 +87,6 @@ ensure_python_initialized <- function(required_module = NULL) {
   if (is.function(callback))
     callback()
 
-  # set up a Python signal handler
-  signals <- import("rpytools.signals")
-  signals$initialize(py_interrupts_pending)
-
-  # register C-level interrupt handler
-  py_register_interrupt_handler()
-
   # call init hooks
   call_init_hooks()
 
@@ -112,7 +106,7 @@ initialize_python <- function(required_module = NULL, use_environment = NULL) {
 
   # provide hint to install Miniconda if no Python is found
   python_not_found <- function(msg) {
-    hint <- "Please create a default virtual environment with `reticulate::virtualenv_create('r-reticulate')`."
+    hint <- 'See the Python "Order of Discovery" here: https://rstudio.github.io/reticulate/articles/versions.html#order-of-discovery.'
     stop(paste(msg, hint, sep = "\n"), call. = FALSE)
   }
 
@@ -174,6 +168,9 @@ initialize_python <- function(required_module = NULL, use_environment = NULL) {
 
   # munge PATH for python (needed so libraries can be found in some cases)
   oldpath <- python_munge_path(config$python)
+  # also munge LD_LIBRARY_PATH on Linux
+  # (needed for Python 3.12 preinstalled on GHA runners, perhaps other installations too)
+  prefix_python_lib_to_ld_library_path(config$python)
 
   # on macOS, we need to do some gymnastics to ensure that Anaconda
   # libraries can be properly discovered (and this will only work in RStudio)
@@ -214,7 +211,8 @@ initialize_python <- function(required_module = NULL, use_environment = NULL) {
                     config$libpython,
                     config$pythonhome,
                     config$virtualenv_activate,
-                    config$version >= "3.0",
+                    config$version$major,
+                    config$version$minor,
                     interactive(),
                     numpy_load_error)
 
@@ -234,6 +232,10 @@ initialize_python <- function(required_module = NULL, use_environment = NULL) {
 
   )
 
+  # allow disabling the Python finalizer
+  if (!tolower(Sys.getenv("RETICULATE_DISABLE_PYTHON_FINALIZER")) %in% c("true", "1", "yes"))
+    reg.finalizer(.globals, function(e) py_finalize(), onexit = TRUE)
+
   # set available flag indicating we have py bindings
   config$available <- TRUE
 
@@ -271,39 +273,13 @@ initialize_python <- function(required_module = NULL, use_environment = NULL) {
   # https://github.com/rstudio/reticulate/issues/586
   py_set_qt_qpa_platform_plugin_path(config)
 
-  # notify the user if the loaded version of Python isn't the same
-  # as the requested version of python
-  local({
-
-    # nothing to do if user didn't request any version
-    requested_versions <- reticulate_python_versions()
-    if (length(requested_versions) == 0)
-      return()
-
-    # if we loaded one of the requested versions, everything is ok
-    actual <- normalizePath(config$python, winslash = "/", mustWork = FALSE)
-    requested <- normalizePath(requested_versions, winslash = "/", mustWork = FALSE)
-    if (actual %in% requested)
-      return()
-
-    # otherwise, warn that we were unable to honor their request
-    if (length(requested_versions) == 1) {
-      fmt <- paste(
-        "Python '%s' was requested but '%s' was loaded instead",
-        "(see reticulate::py_config() for more information)"
-      )
-      msg <- sprintf(fmt, requested_versions[[1]], config$python)
-      warning(msg, call. = FALSE)
-    } else {
-      fmt <- paste(
-        "could not honor request to load desired versions of Python; '%s' was loaded instead",
-        "(see reticulate::py_config() for more information)"
-      )
-      msg <- sprintf(fmt, config$python)
-      warning(msg, call. = FALSE)
+  if (was_python_initialized_by_reticulate()) {
+    allow_threads <- Sys.getenv("RETICULATE_ALLOW_THREADS", "true")
+    allow_threads <- tolower(allow_threads) %in% c("true", "1", "yes")
+    if (allow_threads) {
+      py_allow_threads_impl(TRUE)
     }
-
-  })
+  }
 
   # return config
   config
