@@ -1,11 +1,66 @@
 
 #' @export
 print.python.builtin.object <- function(x, ...) {
-  writeLines(py_repr(x))
+  formatted <- c(
+    py_repr(x),
+    py_format_signature(x)
+  )
+
+  writeLines(formatted)
   invisible(x)
 }
 
 
+py_format_signature <- function(x, ...) {
+  if (!py_is_callable(x))
+    return(NULL)
+
+  inspect <- import("inspect")
+  get_formatted_signature <- function(x, drop_first = FALSE) {
+    tryCatch({
+      sig <- inspect$signature(x)
+      if (drop_first) {
+        # i.e., drop first positional arg, most typically: 'self'
+        #
+        # We only need to do this if inspect.signature() errored on the the
+        # callable itself, but succeeded on callable.__init__. This can happen
+        # for some built-in-C class where methods are slot wrappers.
+        # E.g., builtin exceptions like 'RuntimeError'.
+        sig <- inspect$Signature(
+          parameters = iterate(sig$parameters$values())[-1],
+          return_annotation = sig$return_annotation
+        )
+      }
+
+      formatted <- py_str_impl(sig)
+
+      # split long signatures across multiple lines, so they're readable
+      # if (py_len(sig$parameters) > 5L) {
+      if (nchar(formatted) > 60) {
+        for (formatted_arg in iterate(sig$parameters$values(), py_str_impl))
+          formatted <- sub(formatted_arg,
+                           paste0("\n   ", formatted_arg),
+                           formatted, fixed = TRUE)
+
+        formatted <- sub(", /,", ",\n   /,", formatted, fixed = TRUE) # positional only separator
+        formatted <- sub(", *,", ",\n   *,", formatted, fixed = TRUE) # kw-only separator
+        formatted <- sub("\\)($| ->)", "\n)\\1", formatted) # final closing parens )
+      }
+      formatted
+    },
+    error = function(e) NULL)
+  }
+
+  formatted_sig <- get_formatted_signature(x) %||%
+    get_formatted_signature(py_get_attr(x, "__init__", TRUE), TRUE) %||%
+    get_formatted_signature(py_get_attr(x, "__new__", TRUE), TRUE) %||%
+    "(?)"
+
+  sprintf(" signature: %s", formatted_sig)
+}
+
+
+
 #' @importFrom utils str
 #' @export
 str.python.builtin.object <- function(object, ...) {
@@ -29,31 +84,123 @@ as.character.python.builtin.object <- function(x, ...) {
   py_str(x)
 }
 
-#' Convert Python bytes to an R character vector
+#' Convert Python bytes to an R character or raw vector
 #'
 #' @inheritParams base::as.character
 #'
 #' @param encoding Encoding to use for conversion (defaults to utf-8)
 #' @param errors Policy for handling conversion errors. Default is 'strict'
 #'  which raises an error. Other possible values are 'ignore' and 'replace'.
+#' @param nul Action to take if the bytes contain an embedded NUL (`\x00`).
+#' Python allows embedded `NUL`s in strings, while R does not. There are four
+#' options for handling embedded `NUL`s:
+#'
+#'   1. Error: This is the default
+#'   2. Replace: Supply a replacement string: `nul = "<NUL>"`
+#'   3. Remove: Supply an empty string: `nul = ""`
+#'   4. Split: Supply an R `NULL` to indicate that string should be split at embedded `NUL` bytes: `nul = NULL`
 #'
 #' @export
-as.character.python.builtin.bytes <- function(x, encoding = "utf-8", errors = "strict", ...) {
-  x$decode(encoding = encoding, errors = errors)
+#' @seealso [as.character.python.builtin.str()]
+#' @examplesIf reticulate::py_available()
+#' # A bytes object with embedded NULs
+#' b <- import_builtins(convert = FALSE)$bytes(
+#'   as.raw(c(0x61, 0x20, 0x62, 0x00, 0x63, 0x20, 0x64)) # "a b<NUL>c d"
+#' )
+#'
+#' try(as.character(b))            # Error : Embedded NUL in string.
+#' as.character(b, nul = "<NUL>")  # Replace: "a b<NUL>c d"
+#' as.character(b, nul = "")       # Remove: "a bc d"
+#' as.character(b, nul = NULL)     # Split: "a b" "c d"
+as.character.python.builtin.bytes <-
+  function(x, encoding = "utf-8", errors = "strict",
+           nul = stop("Embedded NUL in string."), ...) {
+    local_conversion_scope(x, TRUE)
+    if(missing(nul))
+      # will throw an error if bytes contain embedded nul
+      x$decode(encoding = encoding, errors = errors)
+
+    else if(is.null(nul)) {
+      # split string at embedded nulls.
+      vapply(x$split(import("builtins")$bytes(list(0L))),
+             function(slice) slice$decode(encoding = encoding, errors = errors),
+             "")
+
+    } else {
+      # replace embedded nulls with supplied string
+      bt <- import("builtins", convert = FALSE)
+      nul <- bt$str(as.character(nul))$encode()
+      x$replace(bt$bytes(list(0L)), nul)$decode(encoding = encoding, errors = errors)
+    }
+
+}
+
+#' @export
+#' @rdname as.character.python.builtin.bytes
+as.raw.python.builtin.bytes <- function(x) {
+  import_builtins()$bytearray(x)
 }
 
 
+#' Convert a Python string to an R Character Vector
+#'
+#' @param x A Python string
+#' @param nul Action to take if the Python string contains an embedded NUL (`\x00`).
+#' Python allows embedded `NUL`s in strings, while R does not. There are four
+#' options for handling embedded `NUL`s:
+#'
+#'   1. Error: This is the default
+#'   2. Replace: Supply a replacement string: `nul = "<NUL>"`
+#'   3. Remove: Supply an empty string: `nul = ""`
+#'   4. Split: Supply an R `NULL` to indicate that string should be split at embedded `NUL` bytes: `nul = NULL`
+#'
+#' @param ... Unused
+#' @export
+#' @return An R character vector. The returned vector will always of length 1,
+#'   unless `nul = NULL` was supplied.
+#' @examplesIf reticulate::py_available()
+#' # Given a Python function that errors when it attempts to return
+#' # a string with an embedded NUL
+#' py_run_string('
+#' def get_string_w_nul():
+#'    return "a b" + chr(0) + "c d"
+#' ')
+#' get_string_w_nul <- py$get_string_w_nul
+#'
+#' try(get_string_w_nul()) # Error : Embedded NUL in string.
+#'
+#' # To get the string into R, use `r_to_py()` on the function to stop it from
+#' # eagerly converting the Python string to R, and then call `as.character()` with
+#' # a `nul` argument supplied to convert the string to R.
+#' get_string_w_nul <- r_to_py(get_string_w_nul)
+#' get_string_w_nul() # unconverted python string: inherits(x, 'python.builtin.str')
+#' as.character(get_string_w_nul(), nul = "<NUL>")  # Replace: "a b<NUL>c d"
+#' as.character(get_string_w_nul(), nul = "")       # Remove: "a bc d"
+#' as.character(get_string_w_nul(), nul = NULL)     # Split: "a b" "c d"
+#'
+#' # cleanup example
+#' rm(get_string_w_nul); py$get_string_w_nul <- NULL
+as.character.python.builtin.str <-
+function(x, nul = stop("Embedded NUL in string."), ...) {
+  if (missing(nul))
+    return(py_str_impl(x))
+
+  local_conversion_scope(x, TRUE)
+  py_nul_str <- import("builtins", convert = FALSE)$chr(0L)
+
+  if (is.null(nul)) # split string at embedded nulls.
+    return(x$split(py_nul_str))
+
+  # else: replace embedded nulls with supplied string
+  x$replace(py_nul_str, as.character(nul))
+
+}
+
 .operators <- new.env(parent = emptyenv())
 
-fetch_op <- function(nm, .op, nargs = 1L) {
-  ensure_python_initialized()
+fetch_op <- function(nm, op, nargs = 1L) {
   if (is.null(fn <- .operators[[nm]])) {
-    force(.op)
-
-    if (is.function(.op))
-      op <- .op
-    else
-      op <- function(...) py_call(.op, ...)
+    force(op)
 
     if (nargs == 1L) {
 
@@ -67,8 +214,8 @@ fetch_op <- function(nm, .op, nargs = 1L) {
         result <- op(...)
         # if either dispatch object has convert=FALSE, don't convert
         convert <-
-          !((inherits(..1, "python.builtin.object") && isFALSE(py_has_convert(..1))) ||
-            (inherits(..2, "python.builtin.object") && isFALSE(py_has_convert(..2))))
+          !((is_py_object(..1) && !py_has_convert(..1)) ||
+            (is_py_object(..2) && !py_has_convert(..2)))
         py_maybe_convert(result, convert)
       }
 
@@ -176,7 +323,6 @@ fetch_op <- function(nm, .op, nargs = 1L) {
 # It will throw an exception on, e.g., with numpy arrays,
 # even though numpy.ndarray defines an __eq__() method.
 py_compare <- function(a, b, op) {
-  ensure_python_initialized()
   py_validate_xptr(a)
   if (!inherits(b, "python.builtin.object"))
     b <- r_to_py(b)
@@ -289,52 +435,13 @@ summary.python.builtin.object <- function(object, ...) {
   str(object)
 }
 
-#' @export
-`$.python.builtin.module` <- function(x, name) {
 
-  # resolve module proxies
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-
-  `$.python.builtin.object`(x, name)
-}
-
-py_has_convert <- function(x) {
-
-  # resolve wrapped environment
-  x <- as.environment(x)
-
-  # get convert flag
-  if (exists("convert", x, inherits = FALSE))
-    get("convert", x, inherits = FALSE)
-  else
-    TRUE
-}
+py_has_convert <- py_get_convert
 
 py_maybe_convert <- function(x, convert) {
-
-  # if this is already an R object, nothing to do
-  if (!inherits(x, "python.builtin.object"))
-    return(x)
-
-  # if it's neither convertable nor callable,
-  # nothing to do
-  convertable <- convert || py_is_callable(x)
-  if (!convertable)
-    return(x)
-
-  # perform conversion
-  # capture previous convert for attr
-  attrib_convert <- py_has_convert(x)
-
-  # temporarily change convert so we can call py_to_r and get S3 dispatch
-  envir <- as.environment(x)
-  assign("convert", convert, envir = envir)
-  on.exit(assign("convert", attrib_convert, envir = envir), add = TRUE)
-
-  # call py_to_r
-  py_to_r(x)
-
+  if(convert)
+    x <- py_to_r(x)
+  x
 }
 
 # helper function for accessing attributes or items from a
@@ -342,22 +449,11 @@ py_maybe_convert <- function(x, convert) {
 # a valid Python object reference
 py_get_attr_or_item <- function(x, name, prefer_attr) {
 
-  # resolve module proxies
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
 
   # skip if this is a NULL xptr
-  if (py_is_null_xptr(x) || !py_available())
+  if (py_is_null_xptr(x))
     return(NULL)
 
-  # special handling for embedded modules (which don't always show
-  # up as "attributes")
-  if (py_is_module(x) && !py_has_attr(x, name)) {
-    module <- py_get_submodule(x, name, py_has_convert(x))
-    if (!is.null(module))
-      return(module)
-  }
-
   # re-cast numeric values as integers
   if (is.numeric(name))
     name <- as.integer(name)
@@ -411,12 +507,29 @@ py_get_attr_or_item <- function(x, name, prefer_attr) {
   py_get_attr_or_item(x, name, FALSE)
 }
 
+#' @export
+`$.python.builtin.module` <- function(x, name) {
+  attr <- py_get_attr(x, name, TRUE)
+  if(!is.null(attr))
+    return(py_maybe_convert(attr, py_has_convert(x)))
 
-# the as.environment generic enables pytyhon objects that manifest
+  # special handling for embedded modules (which don't always show
+  # up as "attributes")
+  module <- py_get_submodule(x, name, py_has_convert(x))
+  if (!is.null(module))
+    return(module)
+
+  # fall back to raising the AttributeError
+  py_get_attr(x, name, FALSE)
+}
+
+# the as.environment generic enables python objects that manifest
 # as R functions (e.g. for functions, classes, callables, etc.) to
-# be automatically converted to enviroments during the construction
-# of PyObjectRef. This makes them a seamless drop-in for standard
-# python objects represented as environments
+# be resolve the environment containing the external pointer (the "refenv")
+# This is still useful e.g., for passing to assign("convert", x, as.environment(x)).
+# This was previously the primary mechanism that allowed for constructing
+# PyObjectRefs from closures, before PyObjectRefs was refactored. The S3 generic
+# is retained for backwards-compatability.
 
 #' @export
 as.environment.python.builtin.object <- function(x) {
@@ -444,13 +557,14 @@ as.environment.python.builtin.object <- function(x) {
 .DollarNames.python.builtin.module <- function(x, pattern = "") {
 
   # resolve module proxies (ignore errors since this is occurring during completion)
-  result <- tryCatch({
-    if (py_is_module_proxy(x))
+  if (py_is_module_proxy(x)) {
+    result <- tryCatch({
       py_resolve_module_proxy(x)
-    TRUE
-  }, error = clear_error_handler(FALSE))
-  if (!result)
-    return(character())
+      TRUE
+    }, error = clear_error_handler(FALSE))
+    if (!result)
+      return(character())
+  }
 
   # delegate
   .DollarNames.python.builtin.object(x, pattern)
@@ -576,8 +690,6 @@ plot.numpy.ndarray <- function(x, y, ...) {
 #' @export
 dict <- function(..., convert = FALSE) {
 
-  ensure_python_initialized()
-
   # get the args
   values <- list(...)
 
@@ -586,7 +698,7 @@ dict <- function(..., convert = FALSE) {
   scan_parent_frame <- TRUE
 
   # if there is a single element and it's a list then use that
-  if (length(values) == 1 && is.null(names(values)) && is.list(values[[1]])) {
+  if (length(values) == 1L && is.null(names(values)) && is.list(values[[1L]])) {
     values <- values[[1]]
     scan_parent_frame <- FALSE
   }
@@ -600,7 +712,7 @@ dict <- function(..., convert = FALSE) {
     # allow python objects to serve as keys
     if (scan_parent_frame && exists(name, envir = frame, inherits = TRUE)) {
       key <- get(name, envir = frame, inherits = TRUE)
-      if (inherits(key, "python.builtin.object"))
+      if (is_py_object(key))
         key
       else
         name
@@ -620,7 +732,6 @@ dict <- function(..., convert = FALSE) {
 #' @rdname dict
 #' @export
 py_dict <- function(keys, values, convert = FALSE) {
-  ensure_python_initialized()
   py_dict_impl(keys, values, convert = convert)
 }
 
@@ -639,16 +750,14 @@ py_dict <- function(keys, values, convert = FALSE) {
 #' @export
 tuple <- function(..., convert = FALSE) {
 
-  ensure_python_initialized()
-
   # get the args
   values <- list(...)
 
   # if it's a single value then maybe do some special resolution
-  if (length(values) == 1) {
+  if (length(values) == 1L) {
 
     # alias value
-    value <- values[[1]]
+    value <- values[[1L]]
 
     # reflect tuples back
     if (inherits(value, "python.builtin.tuple"))
@@ -710,7 +819,10 @@ length.python.builtin.list <- function(x) {
 length.python.builtin.object <- function(x) {
 
   # return 0 if Python not yet available
-  if (py_is_null_xptr(x) || !py_available())
+  # Note: some packages (rgeedim) use `length(module) == 0` as a way to check if
+  # an object is a delayed module without forcing it to load.
+  # Note, a better way to check is: reticulate::py_module_available("module_name")
+  if (py_is_module_proxy(x) || !py_available() || py_is_null_xptr(x))
     return(0L)
 
   # otherwise, try to invoke the object's __len__ method
@@ -765,7 +877,6 @@ py_bool <- function(x) {
 #'
 #' @export
 py_unicode <- function(str) {
-  ensure_python_initialized()
   if (is_python3()) {
     r_to_py(str)
   } else {
@@ -793,8 +904,6 @@ py_unicode <- function(str) {
 #' @export
 with.python.builtin.object <- function(data, expr, as = NULL, ...) {
 
-  ensure_python_initialized()
-
   # enter the context
   context <- data$`__enter__`()
 
@@ -860,64 +969,17 @@ with.python.builtin.object <- function(data, expr, as = NULL, ...) {
 #'
 #' @export
 iterate <- function(it, f = base::identity, simplify = TRUE) {
-
-  ensure_python_initialized()
-
-  # resolve iterator
-  it <- as_iterator(it)
-
-  # perform iteration
-  result <- py_iterate(it, f)
-
-  # simplify if requested and appropriate
-  if (simplify) {
-
-    # attempt to simplify if all elements are length 1
-    lengths <- sapply(result, length)
-    unique_length <- unique(lengths)
-    if (length(unique_length) == 1 && unique_length == 1) {
-
-      # then only simplify if we have a common primitive type
-      classes <- sapply(result, class)
-      unique_class <- unique(classes)
-      if (length(unique_class) == 1 &&
-          unique_class %in% c("character", "complex", "double", "integer", "logical")) {
-        result <- unlist(result)
-      }
-
-    }
-  }
-
-  # return invisibly
-  invisible(result)
+  invisible(py_iterate(it, f, simplify))
 }
 
 
 #' @rdname iterate
 #' @export
 iter_next <- function(it, completed = NULL) {
-
-  # TODO: would like to use PyIter_Check() but that is only implemented
-  # as a macro in Python 2.x and requires copying more headers
-  iterable <- py_has_attr(it, "__next__") || py_has_attr(it, "next")
-  if (!iterable)
-    stop("object is not iterable", call. = FALSE)
-
   py_iter_next(it, completed)
-
 }
 
 
-#' @rdname iterate
-#' @export
-as_iterator <- function(x) {
-  if (inherits(x, "python.builtin.iterator"))
-    x
-  else if (py_has_attr(x, "__iter__"))
-    x$`__iter__`()
-  else
-    stop("iterator function called with non-iterator argument", call. = FALSE)
-}
 
 
 #' Call a Python callable object
@@ -930,63 +992,11 @@ as_iterator <- function(x) {
 #'
 #' @export
 py_call <- function(x, ...) {
-  ensure_python_initialized()
   dots <- split_named_unnamed(list(...))
   py_call_impl(x, dots$unnamed, dots$named)
 }
 
 
-#' Check if a Python object has an attribute
-#'
-#' Check whether a Python object \code{x} has an attribute
-#' \code{name}.
-#'
-#' @param x A python object.
-#' @param name The attribute to be accessed.
-#'
-#' @return \code{TRUE} if the object has the attribute \code{name}, and
-#'   \code{FALSE} otherwise.
-#' @export
-py_has_attr <- function(x, name) {
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-  py_has_attr_impl(x, name)
-}
-
-#' Get an attribute of a Python object
-#'
-#' @param x Python object
-#' @param name Attribute name
-#' @param silent \code{TRUE} to return \code{NULL} if the attribute
-#'  doesn't exist (default is \code{FALSE} which will raise an error)
-#'
-#' @return Attribute of Python object
-#' @export
-py_get_attr <- function(x, name, silent = FALSE) {
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-  res <- py_get_attr_impl(x, name, silent)
-  if(silent && identical(res, emptyenv()))
-    NULL
-  else
-    res
-}
-
-#' Set an attribute of a Python object
-#'
-#' @param x Python object
-#' @param name Attribute name
-#' @param value Attribute value
-#'
-#' @export
-py_set_attr <- function(x, name, value) {
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-  py_set_attr_impl(x, name, value)
-}
 
 #' The Python None object
 #'
@@ -994,22 +1004,9 @@ py_set_attr <- function(x, name, value) {
 #'
 #' @export
 py_none <- function() {
-  ensure_python_initialized()
   py_none_impl()
 }
 
-#' Delete an attribute of a Python object
-#'
-#' @param x A Python object.
-#' @param name The attribute name.
-#'
-#' @export
-py_del_attr <- function(x, name) {
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-  py_del_attr_impl(x, name)
-}
 
 #' List all attributes of a Python object
 #'
@@ -1019,28 +1016,11 @@ py_del_attr <- function(x, name) {
 #' @return Character vector of attributes
 #' @export
 py_list_attributes <- function(x) {
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
   attrs <- py_list_attributes_impl(x)
   Encoding(attrs) <- "UTF-8"
   attrs
 }
 
-py_get_attr_types <- function(x,
-                              names,
-                              resolve_properties = FALSE)
-{
-  ensure_python_initialized()
-  if (py_is_module_proxy(x))
-    py_resolve_module_proxy(x)
-
-  py_get_attr_types_impl(x, names, resolve_properties)
-}
-
-
-
-
 
 #' String representation of a python object.
 #'
@@ -1064,9 +1044,13 @@ py_get_attr_types <- function(x,
 #'
 #' @details The default implementation will call `PyObject_Str` on the object.
 #'
+#' @seealso [as.character.python.builtin.str()]
+#'   [as.character.python.builtin.bytes()] for handling
+#'   `Error : Embedded NUL in string.` if the Python string contains an embedded `NUL`.
+#'
 #' @export
 py_str <- function(object, ...) {
-  if (!inherits(object, "python.builtin.object"))
+  if (!is_py_object(object))
     "<not a python object>"
   else if (py_is_null_xptr(object) || !py_available())
     "<pointer: 0x0>"
@@ -1084,6 +1068,12 @@ py_str.python.builtin.object <- function(object, ...) {
   py_str_impl(object)
 }
 
+#' @export
+format.python.builtin.module <- function(x, ...) {
+  if (py_is_module_proxy(x))
+    return(paste0("Module(", get("module", envir = x), ")", sep = ""))
+  NextMethod()
+}
 
 #' @export
 format.python.builtin.object <- function(x, ...) {
@@ -1216,6 +1206,8 @@ register_class_filter <- function(filter) {
 py_capture_output <- function(expr, type = c("stdout", "stderr")) {
 
   # initialize python if necessary
+  # without expressing an implict venv preference
+  # via an internal import() call
   ensure_python_initialized()
 
   # resolve type argument
@@ -1227,14 +1219,21 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
   # scope output capture
   capture_stdout <- "stdout" %in% type
   capture_stderr <- "stderr" %in% type
-  output_tools$start_capture(capture_stdout, capture_stderr)
-  on.exit(output_tools$end_capture(capture_stdout, capture_stderr), add = TRUE)
 
-  # evaluate the expression
-  force(expr)
+  context_manager <- output_tools$OutputCaptureContext(
+    capture_stdout, capture_stderr
+  )
+
+  context_manager$`__enter__`()
+  tryCatch(
+    force(expr),
+    finally = {
+      context_manager$`__exit__`()
+    }
+  )
 
   # collect output
-  output_tools$collect_output()
+  context_manager$collect_output()
 
 }
 
@@ -1263,8 +1262,6 @@ py_capture_output <- function(expr, type = c("stdout", "stderr")) {
 #'
 #' @export
 py_run_string <- function(code, local = FALSE, convert = TRUE) {
-  ensure_python_initialized()
-  on.exit(py_flush_output(), add = TRUE)
   invisible(py_run_string_impl(code, local, convert))
 }
 
@@ -1321,7 +1318,6 @@ py_run_file <- function(file, local = FALSE, convert = TRUE, prepend_path = TRUE
 #'
 #' @export
 py_eval <- function(code, convert = TRUE) {
-  ensure_python_initialized()
   py_eval_impl(code, convert)
 }
 
@@ -1334,10 +1330,9 @@ py_ellipsis <- function() {
 }
 
 #' @importFrom rlang list2
-py_callable_as_function <- function(callable, convert) {
+py_callable_as_function <- function(callable) {
 
   force(callable)
-  force(convert)
 
   as.function.default(c(py_get_formals(callable), quote({
     cl <- sys.call()
@@ -1346,7 +1341,7 @@ py_callable_as_function <- function(callable, convert) {
     call_args <- split_named_unnamed(eval(cl, parent.frame()))
     result <- py_call_impl(callable, call_args$unnamed, call_args$named)
 
-    if (convert)
+    if(py_get_convert(callable))
       result <- py_to_r(result)
 
     if (is.null(result))
@@ -1371,16 +1366,22 @@ py_is_module <- function(x) {
 }
 
 py_is_module_proxy <- function(x) {
-  inherits(x, "python.builtin.module") && exists("module", envir = x)
+  typeof(x) == "environment" &&
+  exists("module", envir = x, inherits = FALSE) &&
+  inherits(x, "python.builtin.module")
 }
 
 py_resolve_module_proxy <- function(proxy) {
 
+  if(!py_is_module_proxy(proxy))
+    return(FALSE)
+
   # collect module proxy hooks
-  collect_value <- function(name) {
+  collect_value <- function(name, clear = TRUE) {
     if (exists(name, envir = proxy, inherits = FALSE)) {
       value <- get(name, envir = proxy, inherits = FALSE)
-      remove(list = name, envir = proxy)
+      if (clear)
+        remove(list = name, envir = proxy)
       value
     } else {
       NULL
@@ -1395,12 +1396,8 @@ py_resolve_module_proxy <- function(proxy) {
   # get module name
   module <- get("module", envir = proxy)
 
-  # load and error handlers
-  before_load <- collect_value("before_load")
-  on_load <- collect_value("on_load")
-  on_error <- collect_value("on_error")
-
   # execute before load handler
+  before_load <- collect_value("before_load", clear = TRUE)
   if (is.function(before_load))
     before_load()
 
@@ -1408,6 +1405,8 @@ py_resolve_module_proxy <- function(proxy) {
   # python configuration information if we have it
   result <- tryCatch(import(module), error = clear_error_handler())
   if (inherits(result, "error")) {
+    # load and error handlers
+    on_error <- collect_value("on_error", clear = FALSE)
     if (!is.null(on_error)) {
 
       # call custom error handler
@@ -1425,15 +1424,23 @@ py_resolve_module_proxy <- function(proxy) {
     }
   }
 
-  # fixup the proxy
-  py_module_proxy_import(proxy)
-
+  # clear any custom 'on_error' hook
+  collect_value("on_error", clear = TRUE)
   # clear the global tracking of delay load modules
   .globals$delay_load_imports <- NULL
 
+  # fixup the proxy. Note, the proxy may have already been fixed up,
+  # if `import(module)` triggered hooks to run registered via
+  # (unexported) py_register_load_hook()
+  py_module_proxy_import(proxy)
+
+
   # call on_load if provided
+  on_load <- collect_value("on_load", clear = TRUE)
   if (is.function(on_load))
     on_load()
+
+  TRUE
 }
 
 py_get_name <- function(x) {
@@ -1529,23 +1536,16 @@ py_inject_hooks <- function() {
 
   builtins <- import_builtins(convert = TRUE)
 
-  input <- function(prompt = "") {
-
-    response <- tryCatch(
-      readline(prompt),
-      interrupt = identity
-    )
-
-    if (inherits(response, "interrupt"))
-      stop("KeyboardInterrupt", call. = FALSE)
-
-    r_to_py(response)
-
-  }
-
   # override input function
   if (interactive() && was_python_initialized_by_reticulate()) {
+    # PyOS_ReadlineFunctionPointer() is not part of the stable ABI.
+    # PyOS_InputHook() only has one slot - used by other thigns like tkinter.
+    input <- function(prompt = "") {
+      readline(prompt)
+    }
+
     name <- if (is_python3()) "input" else "raw_input"
+    .globals$og_input_builtin <- builtins[[name]]
     builtins[[name]] <- input
   }
 
@@ -1577,9 +1577,12 @@ py_module_onload <- function(module) {
 }
 
 py_module_loaded <- function(module) {
-  sys <- import("sys", convert = TRUE)
-  modules <- sys$modules
-  module %in% names(modules)
+  if(is_python_initialized()) {
+    sys <- import("sys", convert = TRUE)
+    modules <- names(sys$modules)
+  } else
+    modules <- NULL
+  module %in% modules
 }
 
 py_register_load_hook <- function(module, hook) {
@@ -1614,8 +1617,8 @@ py_register_load_hook <- function(module, hook) {
 #' }
 nameOfClass.python.builtin.type <- function(x) {
   paste(
-    as_r_value(py_get_attr_impl(x, "__module__")),
-    as_r_value(py_get_attr_impl(x, "__name__")),
+    as_r_value(py_get_attr(x, "__module__")),
+    as_r_value(py_get_attr(x, "__name__")),
     sep = "."
   )
 }
@@ -1637,9 +1640,6 @@ chooseOpsMethod.python.builtin.object <- function(x, y, mx, my, cl, reverse) {
   identical(environment(my), parent.env(environment()))
 }
 
-py_set_interrupt <- function() {
-  py_set_interrupt_impl()
-}
 
 #' @export
 format.python.builtin.traceback <- function(x, ..., limit = NULL) {
@@ -1661,14 +1661,23 @@ py_clear_last_error <- function() {
 #' @return For `py_last_error()`, `NULL` if no error has yet been encountered.
 #'   Otherwise, a named list with entries:
 #'
-#'   +  `"type"`: R string, name of the exception class.
+#' +  `"type"`: R string, name of the exception class.
 #'
-#'   +  `"value"`: R string, formatted exception message.
+#' +  `"value"`: R string, formatted exception message.
+#'
+#' +  `"traceback"`: R character vector, the formatted python traceback,
+#'
+#' +  `"message"`: The full formatted raised exception, as it would be printed in
+#' Python. Includes the traceback, type, and value.
+#'
+#' +  `"r_trace"`: A `data.frame` with class `rlang_trace` and columns:
+#'
+#'    - `call`: The R callstack, `full_call`, summarized for pretty printing.
+#'    - `full_call`: The R callstack. (Output of `sys.calls()` at the error callsite).
+#'    - `parent`: The parent of each frame in callstack. (Output of `sys.parents()` at the error callsite).
+#'    - Additional columns for internals use: `namespace`, `visible`, `scope`.
 #'
-#'   +  `"traceback"`: R character vector, the formatted python traceback,
 #'
-#'   +  `"message"`: The full formatted raised exception, as it would be printed in
-#'   Python. Includes the traceback, type, and value.
 #'
 #' And attribute `"exception"`, a `'python.builtin.Exception'` object.
 #'
@@ -1677,6 +1686,13 @@ py_clear_last_error <- function() {
 #'
 #' @examples
 #' \dontrun{
+#'
+#' # see last python exception with R traceback
+#' reticulate::py_last_error()
+#'
+#' # see the full R callstack from the last Python exception
+#' reticulate::py_last_error()$r_trace$full_call
+#'
 #' # run python code that might error,
 #' # without modifying the user-visible python exception
 #'
@@ -1733,8 +1749,8 @@ py_last_error <- function(exception) {
     return(NULL)
   }
 
-  etype <- py_get_attr_impl(e, "__class__")
-  etb <- py_get_attr_impl(e, "__traceback__", TRUE)
+  etype <- py_get_attr(e, "__class__")
+  etb <- py_get_attr(e, "__traceback__", TRUE)
   traceback <- import("traceback")
 
   if(is.null(etb))
@@ -1743,7 +1759,7 @@ py_last_error <- function(exception) {
     formatted_traceback <- traceback$format_tb(etb)
 
   out <- list(
-    type = py_get_attr_impl(etype, "__name__", TRUE),
+    type = py_get_attr(etype, "__name__", TRUE),
     value = py_str_impl(e),
     traceback = formatted_traceback,
     message = paste0(traceback$format_exception(etype, e, etb),
@@ -1826,6 +1842,8 @@ print.py_error <- function(x, ...) {
 
   cat_h1("R Traceback")
   print(x$r_trace)
+
+  cat(.py_last_error_full_callstack_hint(), "\n", sep = "")
 }
 
 cat_h1 <- function(x) {
@@ -1884,3 +1902,23 @@ format_py_exception_traceback_with_clickable_filepaths <- function(etb) {
 
   cli::col_silver(paste("Run", py_last_error, "for details."))
 }
+
+
+.py_last_error_full_callstack_hint <- function() {
+
+  hint <- "See `reticulate::py_last_error()$r_trace$full_call` for more details."
+
+  if(!interactive() ||
+     !identical(.Platform$GUI, "RStudio") ||
+     !requireNamespace("cli", quietly = TRUE))
+    return(hint)
+
+  # # ide:run: / rstudio:run: links don't support expressions like this.
+  # last_error_unsummarized_callstack <- cli::style_hyperlink(
+  #   "`reticulate::py_last_error()$r_trace$full_call`",
+  #     "rstudio:run:reticulate::py_last_error()$r_trace$full_call")
+  # hint <- cli::col_silver(paste("See", last_error_unsummarized_callstack,
+  #                               "for more details."))
+
+  cli::col_silver(hint)
+}
