@@ -89,6 +89,18 @@ repl_python <- function(
     on.exit(teardown(), add = TRUE)
   }
 
+
+  ensure_python_initialized()
+  if (is_positron() &&
+      exists(".ps.reticulate_open", inherits = TRUE)) {
+
+    eval(call(".ps.reticulate_open", input))
+
+    # TODO: seems we need to rerun py_inject_r(), possibly other init hooks.
+    # TODO: kernal initializion drops pre-existing objects in __main__
+    return(invisible())
+  }
+
   # split provided code on newlines
   if (!is.null(input))
     input <- unlist(strsplit(input, "\n", fixed = TRUE))
@@ -234,7 +246,7 @@ repl_python <- function(
     if (buffer$empty()) {
 
       # handle user requests to quit
-      if (trimmed %in% c("quit", "exit")) {
+      if (trimmed %in% c("quit", "exit", "quit()", "exit()")) {
         quit_requested <<- TRUE
         return()
       }
