@@ -5,7 +5,7 @@
 #'
 #' Virtual environments are by default located at `~/.virtualenvs` (accessed
 #' with the `virtualenv_root()` function). You can change the default location
-#' by defining the `WORKON_HOME` environment variable.
+#' by defining the `RETICULATE_VIRTUALENV_ROOT` or `WORKON_HOME` environment variables.
 #'
 #' Virtual environments are created from another "starter" or "seed" Python
 #' already installed on the system. Suitable Pythons installed on the system are
@@ -131,6 +131,7 @@ virtualenv_create <- function(
     python <- virtualenv_starter(version)
 
 
+  local_prefix_python_lib_to_ld_library_path(python)
   check_can_be_virtualenv_starter(python, version)
 
   module <- module %||% virtualenv_module(python)
@@ -333,6 +334,10 @@ virtualenv_list <- function() {
 #' @rdname virtualenv-tools
 #' @export
 virtualenv_root <- function() {
+  root <- Sys.getenv("RETICULATE_VIRTUALENV_ROOT", NA_character_)
+  if (!is.na(root))
+    return(root)
+
   Sys.getenv("WORKON_HOME", unset = "~/.virtualenvs")
 }
 
@@ -446,7 +451,7 @@ virtualenv_module <- function(python) {
     if (python_has_module(python, "easy_install")) {
       commands$push(paste("$", python, "-m easy_install --upgrade --user pip"))
     } else if (is_ubuntu() && dirname(python) == "/usr/bin") {
-      package <- if (py_version < 3) "python-pip" else "python3-pip"
+      package <- if (py_version < "3") "python-pip" else "python3-pip"
       commands$push(paste("$ sudo apt-get install", package))
     } else {
       commands$push("$ curl https://bootstrap.pypa.io/get-pip.py -o get-pip.py")
@@ -458,8 +463,8 @@ virtualenv_module <- function(python) {
   # then, recommend installation of virtualenv or venv with pip
   commands$push(paste("Install", modules[[1]], "with:"))
   if (is_ubuntu() && dirname(python) == "/usr/bin") {
-    package <- if (py_version < 3) "python-virtualenv" else "python3-venv"
-    commands$push(paste("$ sudo apt-get install", package))
+    packages <- if (py_version < "3") "python-virtualenv" else "python3-venv python3-pip python3-dev"
+    commands$push(paste("$ sudo apt-get install", packages))
   } else {
     commands$push(paste("$", python, "-m pip install --upgrade --user", module))
   }
@@ -533,18 +538,30 @@ virtualenv_starter <- function(version = NULL, all = FALSE) {
                suffix)
     }
 
-    p <- unique(normalizePath(Sys.glob(glob), winslash = "/"))
+    p <- unique(normalizePath(Sys.glob(glob), winslash = "/", mustWork = FALSE))
     p <- p[grep("^python[0-9.]*(\\.exe)?$", basename(p))]
+    p <- p[utils::file_test("-x", p)]
+    p <- p[utils::file_test("-f", p)]
     v <- numeric_version(vapply(p, function(python_path)
       tryCatch({
+        env <- character()
+        if (is_linux()) {
+          libpath <- file.path(dirname(dirname(python_path)), "lib")
+          if (file.exists(libpath)) {
+            ldpath <- paste(libpath, Sys.getenv("LD_LIBRARY_PATH"), sep = ":")
+            env <- paste("LD_LIBRARY_PATH=", ldpath, sep = "")
+          }
+        }
         v <- suppressWarnings(system2(
           python_path, "-EV",
-          stdout = TRUE, stderr = TRUE))
+          stdout = TRUE, stderr = TRUE,
+          env = env))
         # v should be something like "Python 3.10.6"
         if ((attr(v, "status") %||% 0) ||
             length(v) != 1L ||
             !startsWith(v, "Python "))
           return(NA_character_)
+        v <- sub("\\+.*$", "", v)
         substr(v, 8L, 999L)
       }, error = function(e) NA_character_), ""), strict = FALSE)
     df <- data.frame(version = v, path = p,
@@ -642,6 +659,11 @@ virtualenv_starter <- function(version = NULL, all = FALSE) {
     rownames(starters) <- NULL
   }
 
+  if (is_windows()) {
+    # Windows App Store Python should never be used.
+    starters <- starters[!is_windows_app_store_python(starters$path), ]
+  }
+
   if (all)
     starters
   else if (nrow(starters))
@@ -670,12 +692,14 @@ as_version_constraint_checkers <- function(version) {
   ver <- numeric_version(ver)
 
   .mapply(function(op, ver) {
-    op <- get(op, mode = "function")
+    op <- as.symbol(op)
     force(ver)
 
     # return a "checker" function that takes a vector of versions and returns
     # a logical vector of if the version satisfies the constraint.
-    function(x) {
+    rlang::zap_srcref(eval(bquote(function(x) {
+      op <- .(op)
+      ver <- .(ver)
       x <- numeric_version(x)
       # if the constraint version is missing minor or patch level, set
       # to 0, so we can match on all, equivalent to pip style syntax like '3.8.*'
@@ -685,7 +709,7 @@ as_version_constraint_checkers <- function(version) {
           x[, lvl] <- 0L
         }
       op(x, ver)
-    }
+    })))
   }, list(op, ver), NULL)
 }
 
@@ -710,7 +734,7 @@ can_be_virtualenv_starter <- function(python) {
 
   py2_modules <- c("pip", "virtualenv")
   py3_modules <- c("pip", "venv")
-  modules <- ifelse(version < 3, py2_modules, py3_modules)
+  modules <- ifelse(version < "3", py2_modules, py3_modules)
 
   # ensure these modules are available
   if (!python_has_modules(python, modules))
@@ -756,7 +780,7 @@ stop_no_virtualenv_starter <- function(version = NULL, python = NULL) {
       file.exists(python) &&
       !python_has_modules(python, c("pip", "venv"))) {
     if (is_ubuntu())
-      w("- sudo apt install python3-venv")
+      w("- sudo apt install python3-venv python3-pip python3-dev")
     else if (is_fedora())
       w("- sudo dnf install python3-pip")
     else
