@@ -40,6 +40,9 @@ namespace {
 volatile sig_atomic_t s_pollingRequested;
 bool s_flush_std_buffers = true;
 
+bool running = true;
+tthread::thread* t = nullptr;
+
 // Forward declarations
 int pollForEvents(void*);
 
@@ -49,10 +52,7 @@ int pollForEvents(void*);
 // the polling signal will not be set).
 void eventPollingWorker(void *) {
 
-  while (true) {
-
-    // Throttle via sleep
-    tthread::this_thread::sleep_for(tthread::chrono::milliseconds(200));
+  while (running) {
 
     // Schedule polling on the main thread if the interpeter is still running.
     // Note that Py_AddPendingCall is documented to be callable from a background
@@ -65,6 +65,9 @@ void eventPollingWorker(void *) {
       Py_AddPendingCall(pollForEvents, NULL);
     }
 
+    // Throttle via sleep
+    tthread::this_thread::sleep_for(tthread::chrono::milliseconds(500));
+
   }
 
 }
@@ -82,6 +85,15 @@ void processEvents(void* data) {
 int pollForEvents(void*) {
 
   DBG("Polling for events.");
+  // Some guarantees for code that runs in this function scope:
+  //
+  // 1. We are on the main thread, we have the Python GIL, no other code is
+  // concurrently accessing R. i.e., we can safely use the full Python and R
+  // APIs.
+  //
+  // 2. The Python interpreter is currently on a byte code boundary
+  // (R is waiting on Python to finish). This mean that it is *not* safe to
+  // raise an Exception, risk an R/C lngjmp, or throw a C++ exception.
 
   // Request that the background thread schedule us to be called again
   // (this is delegated to a background thread so that these requests
@@ -104,6 +116,22 @@ int pollForEvents(void*) {
     R_ToplevelExec(processEvents, NULL);
   }
 
+  // Check if we need to set a python interrupt.
+  // This is a fallback in case:
+  //
+  // 1. User code overwrote the C handler reticulate registered and we received
+  // a SIGINT. This can easily happen if python code called signal.signal()
+  //
+  // 2. An R interrupt was only simulated by calling R_onintr() directly (e.g,
+  // rlang::interrupt()), and not actually received as an OS process signal.
+  //
+  // As a fail-safe for the C handler not being called, we also ensure here, in
+  // the background polling worker, that if R_interrupts_pending=1, then a Python
+  // interrupt is pending too.
+  if(reticulate::signals::getInterruptsPending()) {
+    PyErr_SetInterrupt();
+  }
+
   // Success!
   return 0;
 
@@ -113,9 +141,24 @@ int pollForEvents(void*) {
 
 // Initialize event loop polling background thread
 void initialize() {
-  tthread::thread t(eventPollingWorker, NULL);
-  t.detach();
+  running = true;
+  t = new tthread::thread(eventPollingWorker, NULL);
 }
 
+void deinitialize(bool wait) {
+  // signal the thread to stop
+  running = false;
+
+  // clear the ref
+  if (t) {
+    if (wait) { // default: false
+      t->join();
+      delete t;  // Clean up the thread object
+      t = nullptr;
+    }
+  }
+}
+
+
 } // namespace event_loop
 } // namespace reticulate
