@@ -56,21 +56,41 @@ test_that("R datetimes can be passed to Python functions", {
 })
 
 test_that("timezone information is not lost during conversion", {
-  
+
+  skip_if_no_python()
+  skip_if(py_version() < "3")
+
   datetime <- import("datetime", convert = FALSE)
   if (!py_has_attr(datetime, "timezone"))
     skip("datetime.timezone is not available")
-  
+
   pdt <- datetime$datetime(
     year   = 2020L,
     month  = 8L,
     day    = 24L,
+    hour   = 3L,
+    minute = 4L,
+    second = 5L,
     tzinfo = datetime$timezone$utc
   )
-  
+
   rdt <- py_to_r(pdt)
-  
+
   tzone <- attr(rdt, "tzone", exact = TRUE)
   expect_identical(tzone, "UTC")
-  
+
+  expect_identical(
+    format(rdt, "%Y-%m-%dT%H:%M:%S%z"),
+    "2020-08-24T03:04:05+0000")
+  # py_to_r(pdt$isoformat(timespec="seconds")))
+
+  if (py_version() >= "3.9") {
+    zoneinfo <- import("zoneinfo")
+    pdt <- datetime$datetime$now(zoneinfo$ZoneInfo("America/New_York"))
+    rdt <- py_to_r(pdt)
+    expect_identical(attr(rdt, "tzone", TRUE), "America/New_York")
+    expect_identical(format(rdt, "%Y-%m-%dT%H:%M:%S%z"),
+                     py_to_r(pdt$strftime("%Y-%m-%dT%H:%M:%S%z")))
+  }
+
 })
